<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Mage 246 Graphql Fix
 */
declare(strict_types=1);

namespace Amasty\Mage246GraphqlFix\Model\Resolver\Products\DataProvider\Deferred;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\CatalogGraphQl\Model\Resolver\Products\DataProvider\Deferred\Product as OriginalProduct;
use Magento\CatalogGraphQl\Model\Resolver\Products\DataProvider\Product as ProductDataProvider;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\GraphQl\Model\Query\ContextInterface;

class Product extends OriginalProduct
{
    /**
     * @var ProductDataProvider
     */
    private $productDataProvider;

    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @var string[]
     */
    private $productSkus = [];

    /**
     * @var array
     */
    private $productList = [];

    /**
     * @var string[]
     */
    private $attributeCodes = [];

    /**
     * Fix issue with fetching products with visibility "Not Visible Individually"
     *
     * @param ProductDataProvider $productDataProvider
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        ProductDataProvider $productDataProvider,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        $this->productDataProvider = $productDataProvider;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;

        parent::__construct($productDataProvider, $searchCriteriaBuilder);
    }

    /**
     * Add product sku to result set at fetch time.
     *
     * @param string $sku
     * @return void
     */
    public function addProductSku(string $sku) : void
    {
        if (!in_array($sku, $this->productSkus)) {
            $this->productSkus[] = $sku;
        }
    }

    /**
     * Add product skus to result set at fetch time.
     *
     * @param array $skus
     * @return void
     */
    public function addProductSkus(array $skus) : void
    {
        foreach ($skus as $sku) {
            $this->addProductSku($sku);
        }
    }

    /**
     * Add attributes to collection filter
     *
     * @param array $attributeCodes
     * @return void
     */
    public function addEavAttributes(array $attributeCodes) : void
    {
        $this->attributeCodes = array_unique(array_merge($this->attributeCodes, $attributeCodes));
    }

    /**
     * @param string $sku
     * @param ContextInterface|null $context
     * @return array
     */
    public function getProductBySku(string $sku, ContextInterface $context = null) : array
    {
        if (isset($this->productList[$sku])) {
            return $this->productList[$sku];
        }

        $this->fetch($context);

        if (!isset($this->productList[$sku])) {
            return [];
        }

        return $this->productList[$sku];
    }

    /**
     * Fetch product data and return in array format. Keys for products will be their skus.
     *
     * @param null|ContextInterface $context
     */
    private function fetch(ContextInterface $context = null): void
    {
        if (empty($this->productSkus)) {
            return;
        }

        $skusToFetch = array_diff($this->productSkus, array_keys($this->productList));

        if (empty($skusToFetch)) {
            return;
        }

        $this->searchCriteriaBuilder->addFilter(ProductInterface::SKU, $skusToFetch, 'in');
        $result = $this->productDataProvider->getList(
            $this->searchCriteriaBuilder->create(),
            $this->attributeCodes,
            false,
            false, // this fix for issue with fetching products with visibility "Not Visible Individually"
            $context
        );

        /** @var \Magento\Catalog\Model\Product $product */
        foreach ($result->getItems() as $product) {
            $this->productList[$product->getSku()] = ['model' => $product];
        }
    }
}
